unit NewColor;
interface
uses Objects, Drivers, Views, Dialogs, ColorSel;

Type
  PNewColorDialog = ^TNewColorDialog;
  TNewColorDialog = object(TColorDialog)
                      constructor Init(APalette: TPalette; AGroups: PColorGroup);
                    end {TNewColorDialog};

implementation

Type
  PNewColorSelector = ^TNewColorSelector;
  TNewColorSelector = object(TColorSelector)
                        procedure HandleEvent(var Event: TEvent); virtual;
                      end {TNewColorSelector};

  procedure TNewColorSelector.HandleEvent(var Event: TEvent);
  const MaxCol = $F;
        Width  =  4;
  var Mouse: TPoint;
      OldColor: Byte;

      procedure ColorChanged;
      var Msg: Integer;
      begin
        if SelType = csForeground then Msg := cmColorForegroundChanged
                                  else Msg := cmColorBackgroundChanged;
        Message(Owner, evBroadcast, Msg, Pointer(Color))
      end {ColorChanged};

  begin
    case Event.What of

      evMouseDown:
        begin
          OldColor := Color;
          repeat
            if MouseInView(Event.Where) then
            begin
              MakeLocal(Event.Where, Mouse);
              Color := Mouse.Y * 4 + Mouse.X div 3
            end
            else Color := OldColor;
            ColorChanged;
            DrawView
          until not MouseEvent(Event, evMouseMove)
        end;
      evKeyDown:
        begin
          case CtrlToArrow(Event.KeyCode) of

            kbLeft:
              if Color > 0 then Dec(Color)
                           else Color := MaxCol;
            kbRight:
              if Color < MaxCol then Inc(Color)
                                else Color := 0;
            kbUp:
              if Color > Width - 1 then Dec(Color, Width)
              else
                if Color = 0 then Color := MaxCol
                             else Inc(Color, MaxCol - Width);
            kbDown:

              if Color < MaxCol - (Width - 1) then Inc(Color, Width)
              else
                if Color = MaxCol then Color := 0
                                  else Dec(Color, MaxCol - Width);
            else
            begin
              inherited HandleEvent(Event);
              Exit
            end
          end
        end;
      evBroadcast:
        if Event.Command = cmColorSet then
        begin
          if SelType = csBackground then Color := Event.InfoByte SHR Width

                                    else Color := Event.InfoByte AND MaxCol;
          DrawView;
          Exit
        end
        else
        begin
          inherited HandleEvent(Event);
          Exit
        end
      else
      begin
        inherited HandleEvent(Event);
        Exit
      end
    end;
    DrawView;
    ColorChanged;
    ClearEvent(Event)
  end {HandleEvent};


  constructor TNewColorDialog.Init(APalette: TPalette; AGroups: PColorGroup);
  var R: TRect;

      P: PView;
  begin
    R.Assign(0, 0, 61, 18);
    TDialog.Init(R, 'Colors');
    Options := Options or ofCentered;
    Pal := APalette;

    R.Assign(18, 3, 19, 14);
    P := New(PScrollBar, Init(R));
    Insert(P);
    R.Assign(3, 3, 18, 14);
    Groups := New(PColorGroupList, Init(R, PScrollBar(P), AGroups));
    Insert(Groups);
    R.Assign(2, 2, 8, 3);
    Insert(New(PLabel, Init(R, '~G~roup', Groups)));

    R.Assign(41, 3, 42, 14);
    P := New(PScrollBar, Init(R));
    Insert(P);
    R.Assign(21, 3, 41, 14);
    P := New(PColorItemList, Init(R, PScrollBar(P), AGroups^.Items));
    Insert(P);
    R.Assign(20, 2, 25, 3);

    Insert(New(PLabel, Init(R, '~I~tem', P)));

    R.Assign(45, 3, 57, 7);
    ForSel := New(PColorSelector, Init(R, csForeground));
    Insert(ForSel);
    Dec(R.A.Y); R.B.Y := R.A.Y+1;
    ForLabel := New(PLabel, Init(R, '~F~oreground', ForSel));
    Insert(ForLabel);

    Inc(R.A.Y, 7); Inc(R.B.Y,10);
    BakSel := New(PNewColorSelector, Init(R, csBackground));
    Insert(BakSel);
    Dec(R.A.Y); R.B.Y := R.A.Y+1;
    BakLabel := New(PLabel, Init(R, '~B~ackground', BakSel));
    Insert(BakLabel);

    R.Assign(3, 15, 34, 16);
    Display := New(PColorDisplay, Init(R, NewStr(' Text')));
    Insert(Display);

    R.Assign(44, 3, 59, 8);
    MonoSel := New(PMonoSelector, Init(R));
    MonoSel^.Hide;
    Insert(MonoSel);
    R.Assign(43, 2, 49, 3);
    MonoLabel := New(PLabel, Init(R, '~C~olor', MonoSel));
    MonoLabel^.Hide;
    Insert(MonoLabel);

    if (AGroups <> nil) and (AGroups^.Items <> nil) then
      Display^.SetColor(Byte(Pal[AGroups^.Items^.Index]));

    R.Assign(36, 15, 46, 17);
    P := New(PButton, Init(R, 'O~k~', cmOk, bfDefault));
    Insert(P);
    R.Assign(48, 15, 58, 17);
    P := New(PButton, Init(R, 'Cancel', cmCancel, bfNormal));
    Insert(P);
    SelectNext(False)

  end {Init};


var SaveExit: pointer;

procedure NewExit; far;
begin
  ExitProc := SaveExit;
  ASM
        mov   AX,$1003
        mov   BL,$01
        int   $10
  end {BlinkEGA};
end {RunExit};


begin
  SaveExit := ExitProc;
  ExitProc := @NewExit;
  ASM
        mov   AX,$1003
        mov   BL,$00
        int   $10
  end {BriteEGA};
end.